from __future__ import print_function
import argparse
import os
import pytsk3
import pyewf
import sys
from utility.pytskutil import TSKUtil

"""
MIT License

Copyright (c) 2017 Chapin Bryce, Preston Miller

Please share comments and questions at:
    https://github.com/PythonForensics/PythonForensicsCookbook
    or email pyforcookbook@gmail.com

Permission is hereby granted, free of charge, to any person obtaining a copy
of this software and associated documentation files (the "Software"), to deal
in the Software without restriction, including without limitation the rights
to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
copies of the Software, and to permit persons to whom the Software is
furnished to do so, subject to the following conditions:

The above copyright notice and this permission notice shall be included in all
copies or substantial portions of the Software.

THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
SOFTWARE.
"""

__authors__ = ["Chapin Bryce", "Preston Miller"]
__date__ = 20170815
__description__ = "Sample usage of pytskutil"


def main(evidence, image_type):

    # Case-insensitive test -- print files in dir (if dir exists)
    tsk_util = TSKUtil(evidence, image_type)
    dir_obj_test4 = tsk_util.query_directory("/WINDOWS/SYSTEM32/CoNfIg")
    if dir_obj_test4 is not None:
        for dir_tuple in dir_obj_test4:
            for fs_object in dir_tuple[1]:
                # Skip ".", ".." or directory entries without a name.
                if not hasattr(fs_object, "info") or not hasattr(fs_object.info, "name") or not hasattr(fs_object.info.name, "name") or fs_object.info.name.name in [".", ".."]:
                    continue

                file_name = fs_object.info.name.name
                file_path = "PARTITION {}{}/{}".format(dir_tuple[0], "/WINDOWS/SYSTEM32/CONFIG", fs_object.info.name.name)
                print("Found File in Dir: {}".format(file_path))

    # This will scan from the specified dir (root in this case) and is equal to the word "SAM" and is case sensitive

    file_obj_test2 = tsk_util.recurse_files("SAM", path="/", equal=True, case=True)
    if file_obj_test2 is not None:
        for part in file_obj_test2:
            for hit_tuple in part:
                print("Found Hit: {}, {}".format(hit_tuple[0], hit_tuple[1]))


if __name__ == "__main__":
    # Command-line Argument Parser
    parser = argparse.ArgumentParser(
        description=__description__,
        epilog="Developed by {} on {}".format(
            ", ".join(__authors__), __date__)
    )
    parser.add_argument("EVIDENCE_FILE", help="Evidence file path")
    parser.add_argument("TYPE", help="Type of Evidence", choices=("raw", "ewf"))
    args = parser.parse_args()

    if os.path.exists(args.EVIDENCE_FILE) and os.path.isfile(args.EVIDENCE_FILE):
        main(args.EVIDENCE_FILE, args.TYPE)
    else:
        print("[-] Supplied input file {} does not exist or is not a file".format(args.EVIDENCE_FILE))
        sys.exit(1)
